################################################################################
# dclib.pl
# Collection of functions
# Copyright  1997-1999 DCScripts All Rights Reserved
# 
# As part of the installation process, you will be asked
# to accept the terms of this Agreement. This Agreement is
# a legal contract, which specifies the terms of the license
# and warranty limitation between you and DCScripts and DCForum.
# You should carefully read this terms agreement as outlined in
# the readme.txt file included with this distribution before
# installing or using this software.  Unless you have a different license
# agreement obtained from DCScripts, installation or use of this software
# indicates your acceptance of the license and warranty limitation terms
# contained in this Agreement. If you do not agree to the terms of this
# Agreement, promptly delete and destroy all copies of the Software.
###################################################################
##  German translation by Lothar Jung for Webster Internet Dienstleistungen
##  http://Webster.de/dcscripts/
###################################################################


#>>>>>>>>>>>>>>> Keine &Auml;nderungen in diesem Script erforderlich <<<<<<<<
#####
# subroutine display_output
# displays html page using a template file
#####

sub display_output {

   my ($templatefile,$r_namespace) = @_;
   my ($template);

	$| = 1;

  # Open template and read in
  
  open(TEMPLATE,"$templatefile") or 
  my_die ("Fehler in dcforumlib::display_output - Die Datei konnte nicht geoeffnet werden: $template_file", $!);
  {
    local($/) = undef;
    $template = <TEMPLATE>;
  }

  close(TEMPLATE);
  
  $template =~ s/\$([A-Z]+)/$r_namespace->{$1}/g;

  print $template;

}

#####
# subroutine return_output
# same as display_output but instead returns
# the text string
#####

sub return_output {

   my ($templatefile,$r_namespace) = @_;
   my ($template);

   # Open template and read in
  
   open(TEMPLATE,"$templatefile") or 
  		my_die ("Fehler in dcforumlib::display_output - Die Datei konnte nicht geoeffnet werden: $template_file", $!);
   {
      local($/) = undef;
      $template = <TEMPLATE>;
   }
   close(TEMPLATE);
  
   $template =~ s/\$([A-Z]+)/$r_namespace->{$1}/g;
   return $template;

}

#####
# subroutine check_datafile
# Checks to see if $datafile exists
# If it doesn't exists, then it creates one
# and places @{$r_variables}
######

sub check_datafile {

   my($datafile,$r_variables) = @_;

   unless (-e $datafile) {
      open(FILE,">$datafile") or
			my_die("Fehler in subroutine check_datafile in dclib.pl: Die Datei konnte nicht geoeffnet werden: $datafile",$!);

      foreach (@{$r_variables}) {
         print FILE "$_\n";
      }

      close(FILE);
      chmod(0666,$datafile);
   }
}

#####
# subroutine check_dir
# Checks to see if a directory exists.
# If it doesn't, it creates one.
#####

sub check_dir {

  local($folder) = @_;
  unless (-e $folder) {
    mkdir($folder,0777);
    chmod(0777,$folder);
  }


}

#####
# subroutine readdata
# Sucks in all the data from $datafile
# and returns reference to the data
####

sub readdata {

   my($datafile) = shift;
   my($r_data) = [];

   # Pull off datafile name and path

   my ($datadir,$file) = pull_off($datafile);

   AuthGetFileLock("$datadir/$file.lock");

   if (open(DATA,"$datafile")) {
   	@$r_data = <DATA>;
	  close(DATA);
		AuthReleaseFileLock ("$datadir/$file.lock");
	}
	else {
  		AuthReleaseFileLock ("$datadir/$file.lock");
		my_die("Fehler in der Subroutine readdata: Die Datei konnte nicht geoeffnet werden: $datafile",$!);
	}

  $r_data;

}

#####
# function pull_off
# Takes full directory path to a file
# and pulls off directory and file name
#####

sub pull_off {

   my($datafile) = shift;
   my @stuff = split(/\//,$datafile);
   my $file = pop(@stuff);
   my $datadir = join("\/",@stuff);

   return ($datadir, $file);
}

#####
#
# subroutine writedata
#
#####

sub writedata {

   my($datafile,$r_rows) = @_;

   my ($datadir,$file) = pull_off($datafile);
	
   AuthGetFileLock("$datadir/$file.lock");

   if (open(DATA,">$datafile")) {
      print DATA @$r_rows;
      close(DATA);
	   chmod(0666,$datafile);
      AuthReleaseFileLock("$datadir/$file.lock");
   }
   else {
      AuthReleaseFileLock("$datadir/$file.lock");
      my_die("Fehler in der Subroutine writedata: Die Datei konnte nicht geoeffnet werden: $datafile",$!);
   }

}

#####
#
# subroutine appenddata
#
#
#####

sub appenddata {

   my($datafile,$row) = @_;

   my ($datadir,$file) = pull_off($datafile);

   AuthGetFileLock("$datadir/$file.lock");

   if (open(DATA,">>$datafile")) {
      print DATA "$row\n";
      close(DATA);
      AuthReleaseFileLock("$datadir/$file.lock");
   }
   else {
      AuthReleaseFileLock("$datadir/$file.lock");
      my_die("Fehler in der Subroutine appenddata: Die Datei konnte nicht geoeffnet werden: $datafile",$!);
   }
}

#####
# subroutine readdata
# Sucks in all the data from $datafile
# and returns reference to the data
####

sub readdata_nolock {

   my($datafile) = @_;
   my($r_data) = [];

   my ($datadir,$file) = pull_off($datafile);

   open(DATA,"$datafile") or
	   my_die("Fehler in der Subroutine readdata_nolock: Die Datei konnte nicht geoeffnet werden: $datafile",$!);
      @{$r_data} = <DATA>;
   close(DATA);

   $r_data;
}

#####
#
# subroutine writedata
#
#####

sub writedata_nolock {

   my($datafile,$r_rows) = @_;

   my ($datadir,$file) = pull_off($datafile);
  
   open(DATA,">$datafile") or
      my_die("Fehler in der Subroutine writedata_nolock: Die Datei konnte nicht geoeffnet werden: $datafile",$!);
      print DATA @$r_rows;
   close(DATA);

   chmod(0666,$datafile);
  
}

#####
#
# subroutine appenddata
#
#
#####

sub appenddata_nolock {

   my($datafile,$row) = @_;

   my ($datadir,$file) = pull_off($datafile);
  
   open(DATA,">>$datafile") or
	   my_die("Fehler in der Subroutine appenddata_nolock: Die Datei konnte nicht geoeffnet werden: $datafile",$!);
      print DATA "$row\n";
    close(DATA);
}

#####
#
# subroutine remove_dir
# removes directory and all its content
#
####

sub remove_dir {

   my($dir) = shift;  
  
   opendir(F,"$dir") or
	   my_die("Fehler in der Subroutine remove_dir: Das Verzeichnis konnte nicht geoeffnet werden: $dir",$!);

   while ($item = readdir(F)) {
	   unlink("$dir/$item");
   }
   closedir(F);
   rmdir("$dir");
   
}

#####
#
# subroutine clean_data
# Remove HTML and control characters
#
#####

sub clean_data {

  my $text = shift;

  $text =~ s/</&lt;/g;
  $text =~ s/>/&gt;/g;
  $text =~ s/\cM//g;
  $text =~ s/\n\n/<p>/g;
  $text =~ s/\n/<br>/g;
  $text =~ s/\r//g;

  return $text;
}


#####
#
# subroutine my_crypt
# encrpytion routine...rather simple one
# 
#####

sub my_crypt {

  my ($arg1, $arg2) = @_;

  if ($platform eq "FREEBSD") {
	  $arg1 = crypt($arg1,substr($arg1,0,2));
  }
  elsif ($platform eq "UNIX") {
	  $arg1 = crypt($arg1,substr($arg2,0,2));
  }

  $arg1;
}

####
#
# subroutine send_mail
# REVISION
# 29 Dec 1998
#   Added Blatmail for NT servers
# 17 Mar 1998 - Version 1.0
#   First release
#
# NOTE:
#   Following variables are global
#   $smtp_server
#
#
# This should change later
####

sub send_mail {

   my ($dir,$mailprog,$from,$to,$subject,$message) = @_;

   # Remove REMOVE_THIS if it exists

   $from = remove_antispam($from);
   $to = remove_antispam($to);

   # Open The Mail Program

   unless ($platform eq 'NT') {  
		open(MAIL,"|$mailprog -t") or
			my_die("Fehler in der Subroutine send_mail: Das Mailprogramm konnte nicht geoeffnet werden: $mailprog",$!);
    	print MAIL "To: $to\n";
    print MAIL "From: $from\n";
    print MAIL "Subject: $subject\n\n";
    print MAIL "$message\n";
		close (MAIL);
	}
	else {
		#prepare mail message to send
		my (@output,$status);
		my $temp_name = get_session_id();
		my $temp_file = "$dir/$temp_name.emn";
		$output[0] = $message;
		writedata($temp_file,\@output);
		$status = `$mailprog $temp_file -s \"$subject\" -f $from -t $to -server $smtp_server`;
		unlink($temp_file); 
	}

}



#####
#
# subroutine: AuthGetFileLock 
#   Usage:
#     &AuthGetFileLock("lockfilename");
# 
#  This routine locks a file so that no one else
#  can get to it while it is being written to.
# 
#  Parameters:
#     $lock_file = filename to use as a temporary lock file
#
#   Output:
#      None.  It checks for the existance of a lock file
#      and if it finds it, it waits until the lock file
#      disappears.  After the lock file is gone, it opens
#      the lock file up itself.
#
#
#####

sub AuthGetFileLock {  

    my ($lock_file) = @_;

	my $flag = 1;

    while($flag) {
		$flag = check_flock($lock_file)
    }

#		if (-e $lock_file && -M $lock_file > $delta) {
#	    unlink($lock_file);
#		}

#    while (-e $lock_file && time < $endtime) {
#        # Do Nothing
#    }

    open(LOCK_FILE, ">$lock_file");    

#    flock(LOCK_FILE, 2); # 2 exclusively locks the file

} # end of AuthGetFileLock

######
#
# subroutine: AuthReleaseFileLock
#   Usage:
#     &AuthReleaseFileLock($lock_file);
#
#  This routine releases the lock file when we are
#  done with the original file.
#
#  Parameters:
#     $lock_file = filename to use as a temporary lock file
#
#   Output:
#      None. Removes the lock file.
# 
#####

sub AuthReleaseFileLock {
    my ($lock_file) = @_;
       

#    flock(LOCK_FILE, 8);

    close(LOCK_FILE);

    unlink($lock_file);

} # end of ReleaseFileLock   



###
#
# subroutine sendCookie
# Sends HTTP-COOKIE to the client
#
###

sub send_cookie {

  my ($key,$value,$expires,$domain) = @_;

  if ($expires ne "") {
		print "Set-Cookie: $key=$value; expires=$expires; path=/; \n";
	}
	else {
		print "Set-Cookie: $key=$value; path=/; \n";
	}


}

###
#
# subroutine getCookie
# Get cookie from the client
# Cookie's name is $cookie
#
###

sub get_cookie {

  my ($cookie) = @_;
  my (@key_value_pairs, $key, $value);
  @key_value_pairs = split (/;\s/, $ENV{'HTTP_COOKIE'});

  foreach (@key_value_pairs) {
    ($key,$value) = split (/=/, $_);
    if ($key eq $cookie) {
      return $value;
    }
  }
  return 0;
}

####
#
# subroutine add_adtispam
#
# Inserts 'REMOVE_THIS' string after @ sign
#
####

sub add_antispam {

	my $email = shift;
	if ($anti_spam eq 'on') {
		unless ($email =~ /REMOVE_THIS/) {
			$email =~ s/(.*)\@(.*)/$1\@REMOVE_THIS$2/;
		}
	}
	return $email;

}

####
#
# subroutine remove_adtispam
#
# Removes 'REMOVE_THIS' string after @ sign
#
####

sub remove_antispam {

	my $email = shift;
	$email =~ s/REMOVE_THIS//g;
	return $email;

}

#####
#
# subroutine compute_time
# returns time marker for keeping
# track of messages read
#
#####

sub compute_time {

	my($year, $month, $day, $hour, $minute, $second) = @_;

	$year += 1900 if ($year < 1000);

   my $mesg_mark = (($year - 1997) * 372 * 86400) +
            ($month * 31 * 86400) + ($day * 86400) +
            ($hour * 3600) + ($minute * 60) + $second;

	return $mesg_mark;
}

####
# subroutine get_date
# modified to work with dcboard
####

sub get_date {

   my ($r_in) = @_;
   my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime(time+3600*$time_offset);

	# Format to look nice
	
   $sec = sprintf("%02d",$sec);
   $min = sprintf("%02d",$min);
   $hour = sprintf("%02d",$hour);
   $mon = sprintf("%02d",$mon+1);
   $mday = sprintf("%02d",$mday);

   $year += 1900;

  my $last_read = (($year - 1997) * 372 * 86400) +
     ($mon * 31 * 86400) + (($mday - $r_in->{'days'}) * 86400)
      + ($hour * 3600) + ($min * 60) + $sec;

  my $time_now = (($year - 1997) * 372 * 86400) +
     ($mon * 31 * 86400) + ($mday * 86400)
      + ($hour * 3600) + ($min * 60) + $sec;

   my $month = $months[$mon];

  my $localtime = "$hour\:$min\:$sec";
  my $date = "$mon/$mday/$year";

  $r_in->{'localtime'} = $localtime;
  $r_in->{'date'} = $date;
  $r_in->{'last_read'} = $last_read;
  $r_in->{'time_now'} = $time_now;

}

####
#
# subroutine my_die
#
####

sub my_die {

	my($my_mesg, $sys_mesg) = @_;
  	print "\n";

	print qq~
	<html>
	<head>
	<title>$my_mesg</title>
	</head>
	<body bgcolor="#FFFFFF">
	<font face="verdana" size="5"><b>SCRIPT ERROR!!!</b></font>
	<hr>
	<font face="verdana" size="3">
	<b>
	Bei diesem Vorgang ist ein Fehler aufgetreten.<BR>
	Folgend ist der Fehler beschrieben.:
	<ul>
	<li>Script Message: $my_mesg
	<li>System Message: $sys_mesg
	</ul>
	<hr>
	Bitte benachrichtigen Sie den <a href="mailto:$admin_email">Administrator</a> der Seite.
	<p>
	Vielen Dank.
	</b>
	</font>

	</body>
	</html>
	~;
	exit;			
}

#####
#
# Subroutine get_session_id
# Generates session ID for a User.
# This portion of code from Selena and Gunther's
# auth-extra-lib.pl
#
####


sub get_session_id {

	my ($session, $session_file);

	# Seed the random generator

	srand($$|time);
	$session = int(rand(60000));

	# pack the time, process id, and random $session into a
	# hex number which will make up the session id.

	$session = unpack("H*", pack("Nnn", time, $$, $session));

} # End of MakeSessionFile

#####
# function time_format
#
#####

sub time_format {
   my ($string) = shift;
	my ($hr,$min,$sec) = split(/\:/,$string);
	($hr,$am_pm) = am_pm($hr);
	$string = join(":",$hr,$min);
	$string .= "&nbsp;" . $am_pm;
	return $string;
}

#####
# function date_format
#
#####

sub date_format {
   my ($string) = shift;
   my @months = qw(Jan Feb Mar Apr Mai Jun Jul Aug Sep Okt Nov Dez);
	my ($mon,$day,$year) = split(/\//,$string);
	if ($year > 100) {
		$year = substr($year,2,2);
	}
	if ($date_format eq "mmm-dd-yy") {
		$string = join("-",$months[$mon-1],$day,$year);
	}
	elsif ($date_format eq "dd-mmm-yy") {
		$string = join("-",$day,$months[$mon-1],$year);
	}
	elsif ($date_format eq "dd-mm-yy") {
		$string = join("-",$day,$mon,$year);
	}
	elsif ($date_format eq "mm-dd-yy") {
		$string = join("-",$mon,$day,$year);
	}

	return $string;
}

#####
# function am_pm
# given hour, print am or pm
####

sub am_pm {
	my ($hour) = shift;
	my $am_pm = 'AM';
	
	if ($hour > 12) {
		$hour -= 12;
		$am_pm = 'PM';
	}
	elsif ($hour == 12) {
		$am_pm = 'PM';
	}

   $hour = sprintf("%02d",$hour);
	return ($hour,$am_pm);
}

#####
# function toggle_color
# for switching back and forth...
#####

sub toggle_color {

	my $reset = shift;

	unless ($reset) {
		if ($bgcolor eq $bg_color_1) {
			$bgcolor = $bg_color_2;
			$font_color = $font_color_2;
			$font_face = $font_face_2;
			$font_size = $font_size_2;
		}
		else {
			$bgcolor = $bg_color_1;
			$font_color = $font_color_1;
			$font_face = $font_face_1;
			$font_size = $font_size_1;
		}
	}
	else {
		$bgcolor = $bg_color_1;
		$font_color = $font_color_1;
		$font_face = $font_face_1;
		$font_size = $font_size_1;
	}
}

#####
# function trim
# trim words that are certain size in length
#####

sub trim {

	my $string = shift;
	my @sentence = split(/\s+/,$string);
	my $j = 0;
	foreach $word (@sentence) {
		my $s_length = length($word);
		if ($s_length > $word_length_max) {
			$sentence[$j] = substr($word,0,$word_length_max-1) . "- " . substr($word,$word_length_max,$s_length)
		}
		$j++;
	}
	$string = join(" ",@sentence);
	return $string;
}

#####
# function html_cntl
# convert html tags to control characters
#####

sub html_cntl {

  my $text = shift;

  $text =~ s/&lt;/>/g;
  $text =~ s/&gt;/>/g;
  $text =~ s/<p>/\n\n/g;
  $text =~ s/<br>/\n/g;
  return $text;

}


####
#
#
####

sub check_flock {

   my ($file) = shift;

   if (-e $file) {
      sleep (1);
      return 1;
   }
   else {
      return 0;
   }

}


1;